#!/usr/bin/env python
# coding: utf-8

#THIS SCRIPT DEPENDS ON PYTHON 2.9, ARCPY AND THE 3D ANALYST EXTENSION. 
#THIS SCRIPT REQUIRES LIDAR DATA AS INPUT AND USER DEFINED PATHWAYS.
#THIS SCRIPT CREATES SMOOTHED BUILDING OBJECTS FROM LIDAR POINT CLOUD DATA.
#CONTACT: ZACKARY WERNER (ZACKARY.WERNER@USDA.GOV)

# Load libraries 
import os
#os.environ["CUDA_VISIBLE_DEVICES"] = "-1"
import pandas as pd
import arcpy
import arcgis
arcpy.CheckOutExtension("3D")
arcpy.env.overwriteOutput = True
print("libs")

#########################
#USER DEFINED PERAMETERS
#########################

# Set Parameters 
outputDir = r'\\166.2.126.25\rseat\Programs\GeoTASC\fy2022\wui_data\prefireLidarAnalysis\waldo'

# Input Data 
in_lasd = r'\\166.2.126.25\rseat\Programs\GeoTASC\fy2022\wui_data\WaldoFire_CO2012\2011_prefire_lidar\waldoLidarSubset.lasd'

#the projection you want the final product in
proj = 'PROJCS["NAD_1983_HARN_StatePlane_Colorado_Central_FIPS_0502_Feet",GEOGCS["GCS_North_American_1983_HARN",DATUM["D_North_American_1983_HARN",SPHEROID["GRS_1980",6378137.0,298.257222101]],PRIMEM["Greenwich",0.0],UNIT["Degree",0.0174532925199433]],PROJECTION["Lambert_Conformal_Conic"],PARAMETER["False_Easting",3000000.000316083],PARAMETER["False_Northing",999999.999996],PARAMETER["Central_Meridian",-105.5],PARAMETER["Standard_Parallel_1",38.45],PARAMETER["Standard_Parallel_2",39.75],PARAMETER["Latitude_Of_Origin",37.83333333333334],UNIT["Foot_US",0.3048006096012192]],VERTCS["NAVD88 - Geoid96 (Feet)",VDATUM["unknown"],PARAMETER["Vertical_Shift",0.0],PARAMETER["Direction",1.0],UNIT["Foot_US",0.3048006096012192]]'

#########################
#USER DEFINED PERAMETERS
#########################

# Prep the output dir 
if not os.path.exists(os.path.join(outputDir, "working")):
            os.makedirs(os.path.join(outputDir, "working"))
        
if not os.path.exists(os.path.join(outputDir, "final")):
            os.makedirs(os.path.join(outputDir, "final"))

#DONT RUN IF BUILDINGS ARE ALREADY CLASSIFED IN THE POINT CLOUD
# Classify buildings in lasd
buildingClassified = arcpy.ddd.ClassifyLasBuilding(in_lasd, min_height='3 meters', min_area='4 SquareMeters')
print("ClassifyLasBuilding")

# Make a layer based on building classification only 
buildingPC = arcpy.management.MakeLasDatasetLayer(buildingClassified, "Building_layer", "6", "'Last Return';'Single Return';'First of Many';'Last of Many';1;2;3;4;5;6;7;8;9;10;11;12;13;14;15", "INCLUDE_UNFLAGGED", "INCLUDE_SYNTHETIC", "INCLUDE_KEYPOINT", "EXCLUDE_WITHHELD", None, "INCLUDE_OVERLAP")
print("MakeLasDatasetLayer")

# Create building 1m (3ft) raster
buildingRast = arcpy.management.LasPointStatsAsRaster(buildingPC,  os.path.join(outputDir,'working\Building_raster.tif'), "PREDOMINANT_CLASS", "CELLSIZE", 3)
print("LasPointStatsAsRaster")

#Aggregate building polygons to 2m
#DONT RUN IF 1m (3ft) IS THE APPROPRIATE RESOLUTION FOR YOUR RASTER
with arcpy.EnvManager(outputCoordinateSystem=proj, parallelProcessingFactor="100%"):
    buildingRast = arcpy.sa.Aggregate(buildingRast, 2, "MAXIMUM", "EXPAND", "DATA"); buildingRast.save(os.path.join(outputDir,'working\BuildingsAgg.tif'))
print("Aggregate")

# Create building polys
buildingPolys = arcpy.conversion.RasterToPolygon(buildingRast, os.path.join(outputDir, 'working\Buildings_poly'), "NO_SIMPLIFY", "Value", "SINGLE_OUTER_PART", None)
print("buildingAgg")

# Regularize polys 
buildingPolys_Reg = arcpy.ddd.RegularizeBuildingFootprint(buildingPolys, os.path.join(outputDir,'working\Buildings_poly_reg'), "RIGHT_ANGLES", 3, None, 0.25, 1.5, 0.1, 1000000, None, None, "DISTANCE")
print("RegularizeBuildingFootprint")

# Eliminate polygon parts 
buildingPolys_elimin = arcpy.management.EliminatePolygonPart(buildingPolys_Reg, os.path.join(outputDir,'working\Buildings_poly_Elimin'), "AREA", "900 SquareMeters", 0, "CONTAINED_ONLY")
print("EliminatePolygonPart")

# Add area field and calculate geometry
buildingPolys_elimin = arcpy.management.AddField(buildingPolys_elimin, "Area", "DOUBLE")
buildingPolys_elimin = arcpy.management.CalculateGeometryAttributes(buildingPolys_elimin, "Area AREA", '',
                                                                    "SQUARE_FEET_US", proj, "SAME_AS_INPUT")
print("CalculateGeometryAttributes")

#Select all buildings greater than 120 sq.ft. and write that file 
buidlingSelect = arcpy.management.SelectLayerByAttribute(buildingPolys_elimin, "NEW_SELECTION", '"Area" > 120')

print("writing final file...")
buildingsPolys_Final = arcpy.management.CopyFeatures(buidlingSelect, os.path.join(outputDir,'final\Buildings_final'))
print(buildingsPolys_Final)
